#include <Imagine/Graphics.h>
#include <cstdlib>
#include "grille.h"
using namespace Imagine;
using namespace std;

// Constructeur
Grille::Grille(int w_, int h_) {
    w = w_;
    h = h_;
    grille = new Cellule[w*h];
    for(int j=0; j<h; j++)
        for(int i=0; i<w; i++)
            set(i, j, false); // Initialise la grille vide
}

// Destructeur
Grille::~Grille() {
    delete[] grille;
}

// Setteur pour la grille
void Grille::set(int x, int y, bool val) {
    assert(0<=x && x<w && 0<=y && y<h);
    grille[y*w+x].etat = val;
}

// Change l'etat de la cellule
void Grille::toggle(int x, int y) {
    assert(0<=x && x<w && 0<=y && y<h);
    grille[y*w+x].etat = !grille[y*w+x].etat;
}

bool Grille::get(int x, int y) const {
    assert(0<=x && x<w && 0<=y && y<h);
    return grille[y*w+x].etat;
}

// Dessine toute la grille
void Grille::dessine() const {
    for(int x=0; x<w; x++)
        for(int y=0; y<h; y++) {
            if(get(x,y))
                fillRect(x*z, y*z, z, z, BLACK);
            else
                fillRect(x*z, y*z, z, z, WHITE);
        }
}

// Reinitialise le tableau des changements
void Grille::reset() {
    for(int y=0; y<h; y++)
        for(int x=0; x<w; x++)
            grille[y*w+x].change = false;
}

// La case (x,y) est signalee comme changement
void Grille::change(int x, int y) {
    grille[y*w+x].change = true;
}

// Indique si la case (x,y) a change
bool Grille::get_changement(int x, int y) {
    assert(0<=x && x<w && 0<=y && y<h);
    return grille[y*w+x].change;
}

// Compte le nombre de cellules vivantes dans le 8-voisinage
int Grille::compte_voisins(int x, int y, bool tore) {
    int compte = 0;
    for(int k=x-1; k<=x+1; k++)
        for(int l= y-1; l<=y+1; l++) {
            if(k==x && l==y) continue;
            int vx=k, vy=l;
            if(vx<0) {
                if(tore) vx=w-1;
                else continue;
            }
            if(vx>=w) {
                if(tore) vx=0;
                else continue;
            }
            if(vy<0) {
                if(tore) vy=h-1;
                else continue;
            }
            if(vy>=h) {
                if(tore) vy=0;
                else continue;
            }
            if(get(vx,vy))
                ++compte;
        }
    return compte;
}

void Grille::update(bool tore) {
    reset();
    for(int y=0; y<h; y++)
        for(int x=0; x<w; x++) {
            int compte = compte_voisins(x, y, tore);
            if(get(x,y) && (compte<2 || compte>3))
                change(x,y);
            else if(!get(x,y) && compte==3)
                change(x,y);
        }

    // On applique les changements a la grille
    for(int y=0; y<h; y++)
        for(int x=0; x<w; x++)
            if(get_changement(x,y))
                toggle(x,y);
}

void Grille::ligne(int x, int y) {
    set(x,y,  true);
    set(x,y+1,true);
    set(x,y+2,true);
}

// Indentation speciale du code suivant la forme en question
void Grille::grenouille(int x, int y) {
    set(x,y,  true); set(x+1,y,true); set(x+1,y+1,true);
                     set(x+2,y,true); set(x+2,y+1,true); set(x+3,y+1,true);
}

// Indentation speciale du code suivant la forme en question
void Grille::planeur(int x, int y) {
                       set(x,y,  true);
                                        set(x+1,y+1,true);
    set(x-1,y+2,true); set(x,y+2,true); set(x+1,y+2,true);

}
